@@ -6,12 +6,11 @@ Created on 27.09.2017
 """
 
 import asyncio
+import enum
+import logging
+import re
 
 import aiohttp
-import json
-import logging
-import enum
-import re
 
 _LOGGER = logging.getLogger(__name__)
 DEGREE_CELSIUS = "°C"
@@ -104,7 +103,9 @@ class Fronius:
         api_version  Version of Fronius API to use
     """
 
-    def __init__(self, session, url, api_version=API_VERSION.AUTO):
+    def __init__(
+        self, session: aiohttp.ClientSession, url: str, api_version=API_VERSION.AUTO
+    ):
         """
         Constructor
         """
@@ -122,9 +123,8 @@ class Fronius:
         """
         try:
             async with self._aio_session.get(url) as res:
-                text = await res.text()
-                text = json.loads(text)
-        except aiohttp.ServerTimeoutError:
+                result = await res.json(content_type=None)
+        except (aiohttp.ServerTimeoutError, asyncio.TimeoutError):
             raise ConnectionError(
                 "Connection to Fronius device timed out at {}.".format(url)
             )
@@ -132,9 +132,9 @@ class Fronius:
             raise ConnectionError(
                 "Connection to Fronius device failed at {}.".format(url)
             )
-        except json.JSONDecodeError:
+        except aiohttp.ContentTypeError:
             raise ValueError("Host returned a non-JSON reply at {}.".format(url))
-        return text
+        return result
 
     async def fetch_api_version(self):
         """
@@ -267,12 +267,11 @@ class Fronius:
                 "No header data returned from {} ({})".format(spec, spec_formattings)
             )
         try:
-            # TODO use update here as well
-            fun(sensor, res["Body"]["Data"])
+            sensor.update(fun(res["Body"]["Data"]))
         except (TypeError, KeyError):
             # LoggerInfo oddly deviates from the default scheme
             try:
-                fun(sensor, res["Body"]["LoggerInfo"])
+                sensor.update(fun(res["Body"]["LoggerInfo"]))
             except (TypeError, KeyError):
                 # break if Data is empty
                 _LOGGER.info(
@@ -324,6 +323,15 @@ class Fronius:
             Fronius._device_storage_data, URL_DEVICE_STORAGE, "current storage", device
         )
 
+    async def current_system_storage_data(self):
+        """
+        Get the current storage data for a device.
+        Provides data about batteries.
+        """
+        return await self._current_data(
+            Fronius._system_storage_data, URL_DEVICE_STORAGE, "current system storage"
+        )
+
     async def current_inverter_data(self, device=1):
         """
         Get the current inverter data of one device.
@@ -362,8 +370,9 @@ class Fronius:
         )
 
     @staticmethod
-    def _system_led_data(sensor, data):
+    def _system_led_data(data):
         _LOGGER.debug("Converting system led data: '{}'".format(data))
+        sensor = {}
 
         _map = {
             "PowerLED": "power_led",
@@ -382,8 +391,9 @@ class Fronius:
         return sensor
 
     @staticmethod
-    def _system_power_flow(sensor, data):
+    def _system_power_flow(data):
         _LOGGER.debug("Converting system power flow data: '{}'".format(data))
+        sensor = {}
 
         site = data["Site"]
         # Backwards compatability
@@ -439,19 +449,20 @@ class Fronius:
         return sensor
 
     @staticmethod
-    def _system_meter_data(sensor, data):
+    def _system_meter_data(data):
         _LOGGER.debug("Converting system meter data: '{}'".format(data))
 
-        sensor["meters"] = {}
+        sensor = {"meters": {}}
 
-        for i in data:
-            sensor["meters"][i] = Fronius._meter_data(data[i])
+        for device_id, device_data in data.items():
+            sensor["meters"][device_id] = Fronius._device_meter_data(device_data)
 
         return sensor
 
     @staticmethod
-    def _system_inverter_data(sensor, data):
+    def _system_inverter_data(data):
         _LOGGER.debug("Converting system inverter data: '{}'".format(data))
+        sensor = {}
 
         sensor["energy_day"] = {"value": 0, "unit": WATT_HOUR}
         sensor["energy_total"] = {"value": 0, "unit": WATT_HOUR}
@@ -493,109 +504,9 @@ class Fronius:
         return sensor
 
     @staticmethod
-    def _device_meter_data(sensor, data):
+    def _device_meter_data(data):
         _LOGGER.debug("Converting meter data: '{}'".format(data))
 
-        sensor.update(Fronius._meter_data(data))
-
-        return sensor
-
-    @staticmethod
-    def _device_storage_data(sensor, data):
-        _LOGGER.debug("Converting storage data from '{}'".format(data))
-
-        if "Controller" in data:
-            controller = Fronius._controller_data(data["Controller"])
-            sensor.update(controller)
-
-        if "Modules" in data:
-            sensor["modules"] = {}
-            module_count = 0
-
-            for module in data["Modules"]:
-                sensor["modules"][module_count] = Fronius._module_data(module)
-                module_count += 1
-
-        return sensor
-
-    @staticmethod
-    def _device_inverter_data(sensor, data):
-        _LOGGER.debug("Converting inverter data from '{}'".format(data))
-
-        if "DAY_ENERGY" in data:
-            sensor["energy_day"] = {
-                "value": data["DAY_ENERGY"]["Value"],
-                "unit": data["DAY_ENERGY"]["Unit"],
-            }
-        if "TOTAL_ENERGY" in data:
-            sensor["energy_total"] = {
-                "value": data["TOTAL_ENERGY"]["Value"],
-                "unit": data["TOTAL_ENERGY"]["Unit"],
-            }
-        if "YEAR_ENERGY" in data:
-            sensor["energy_year"] = {
-                "value": data["YEAR_ENERGY"]["Value"],
-                "unit": data["YEAR_ENERGY"]["Unit"],
-            }
-        if "FAC" in data:
-            sensor["frequency_ac"] = {
-                "value": data["FAC"]["Value"],
-                "unit": data["FAC"]["Unit"],
-            }
-        if "IAC" in data:
-            sensor["current_ac"] = {
-                "value": data["IAC"]["Value"],
-                "unit": data["IAC"]["Unit"],
-            }
-        if "IDC" in data:
-            sensor["current_dc"] = {
-                "value": data["IDC"]["Value"],
-                "unit": data["IDC"]["Unit"],
-            }
-        if "IDC_2" in data:
-            sensor["current_dc_2"] = {
-                "value": data["IDC_2"]["Value"],
-                "unit": data["IDC_2"]["Unit"],
-            }
-        if "PAC" in data:
-            sensor["power_ac"] = {
-                "value": data["PAC"]["Value"],
-                "unit": data["PAC"]["Unit"],
-            }
-        if "UAC" in data:
-            sensor["voltage_ac"] = {
-                "value": data["UAC"]["Value"],
-                "unit": data["UAC"]["Unit"],
-            }
-        if "UDC" in data:
-            sensor["voltage_dc"] = {
-                "value": data["UDC"]["Value"],
-                "unit": data["UDC"]["Unit"],
-            }
-        if "UDC_2" in data:
-            sensor["voltage_dc_2"] = {
-                "value": data["UDC_2"]["Value"],
-                "unit": data["UDC_2"]["Unit"],
-            }
-        if "DeviceStatus" in data:
-            if "InverterState" in data["DeviceStatus"]:
-                sensor["inverter_state"] = {
-                    "value": data["DeviceStatus"]["InverterState"]
-                }
-            if "ErrorCode" in data["DeviceStatus"]:
-                sensor["error_code"] = {"value": data["DeviceStatus"]["ErrorCode"]}
-            if "StatusCode" in data["DeviceStatus"]:
-                sensor["status_code"] = {"value": data["DeviceStatus"]["StatusCode"]}
-            if "LEDState" in data["DeviceStatus"]:
-                sensor["led_state"] = {"value": data["DeviceStatus"]["LEDState"]}
-            if "LEDColor" in data["DeviceStatus"]:
-                sensor["led_color"] = {"value": data["DeviceStatus"]["LEDColor"]}
-
-        return sensor
-
-    @staticmethod
-    def _meter_data(data):
-
         meter = {}
 
         if "Current_AC_Phase_1" in data:
@@ -802,6 +713,112 @@ class Fronius:
 
         return meter
 
+    @staticmethod
+    def _device_storage_data(data):
+        _LOGGER.debug("Converting storage data from '{}'".format(data))
+        sensor = {}
+
+        if "Controller" in data:
+            controller = Fronius._controller_data(data["Controller"])
+            sensor.update(controller)
+
+        if "Modules" in data:
+            sensor["modules"] = {}
+            module_count = 0
+
+            for module in data["Modules"]:
+                sensor["modules"][module_count] = Fronius._module_data(module)
+                module_count += 1
+
+        return sensor
+
+    @staticmethod
+    def _system_storage_data(data):
+        _LOGGER.debug("Converting system storage data: '{}'".format(data))
+
+        sensor = {"storages": {}}
+
+        for device_id, device_data in data.items():
+            sensor["storages"][device_id] = Fronius._device_storage_data(device_data)
+
+        return sensor
+
+    @staticmethod
+    def _device_inverter_data(data):
+        _LOGGER.debug("Converting inverter data from '{}'".format(data))
+        sensor = {}
+
+        if "DAY_ENERGY" in data:
+            sensor["energy_day"] = {
+                "value": data["DAY_ENERGY"]["Value"],
+                "unit": data["DAY_ENERGY"]["Unit"],
+            }
+        if "TOTAL_ENERGY" in data:
+            sensor["energy_total"] = {
+                "value": data["TOTAL_ENERGY"]["Value"],
+                "unit": data["TOTAL_ENERGY"]["Unit"],
+            }
+        if "YEAR_ENERGY" in data:
+            sensor["energy_year"] = {
+                "value": data["YEAR_ENERGY"]["Value"],
+                "unit": data["YEAR_ENERGY"]["Unit"],
+            }
+        if "FAC" in data:
+            sensor["frequency_ac"] = {
+                "value": data["FAC"]["Value"],
+                "unit": data["FAC"]["Unit"],
+            }
+        if "IAC" in data:
+            sensor["current_ac"] = {
+                "value": data["IAC"]["Value"],
+                "unit": data["IAC"]["Unit"],
+            }
+        if "IDC" in data:
+            sensor["current_dc"] = {
+                "value": data["IDC"]["Value"],
+                "unit": data["IDC"]["Unit"],
+            }
+        if "IDC_2" in data:
+            sensor["current_dc_2"] = {
+                "value": data["IDC_2"]["Value"],
+                "unit": data["IDC_2"]["Unit"],
+            }
+        if "PAC" in data:
+            sensor["power_ac"] = {
+                "value": data["PAC"]["Value"],
+                "unit": data["PAC"]["Unit"],
+            }
+        if "UAC" in data:
+            sensor["voltage_ac"] = {
+                "value": data["UAC"]["Value"],
+                "unit": data["UAC"]["Unit"],
+            }
+        if "UDC" in data:
+            sensor["voltage_dc"] = {
+                "value": data["UDC"]["Value"],
+                "unit": data["UDC"]["Unit"],
+            }
+        if "UDC_2" in data:
+            sensor["voltage_dc_2"] = {
+                "value": data["UDC_2"]["Value"],
+                "unit": data["UDC_2"]["Unit"],
+            }
+        if "DeviceStatus" in data:
+            if "InverterState" in data["DeviceStatus"]:
+                sensor["inverter_state"] = {
+                    "value": data["DeviceStatus"]["InverterState"]
+                }
+            if "ErrorCode" in data["DeviceStatus"]:
+                sensor["error_code"] = {"value": data["DeviceStatus"]["ErrorCode"]}
+            if "StatusCode" in data["DeviceStatus"]:
+                sensor["status_code"] = {"value": data["DeviceStatus"]["StatusCode"]}
+            if "LEDState" in data["DeviceStatus"]:
+                sensor["led_state"] = {"value": data["DeviceStatus"]["LEDState"]}
+            if "LEDColor" in data["DeviceStatus"]:
+                sensor["led_color"] = {"value": data["DeviceStatus"]["LEDColor"]}
+
+        return sensor
+
     @staticmethod
     def _controller_data(data):
 
@@ -913,8 +930,9 @@ class Fronius:
         return module
 
     @staticmethod
-    def _system_active_device_info(sensor: dict, data: dict):
+    def _system_active_device_info(data: dict):
         _LOGGER.debug("Converting system active device data: '{}'".format(data))
+        sensor = {}
 
         if "Inverter" in data:
             inverters = []
@@ -976,8 +994,9 @@ class Fronius:
         return sensor
 
     @staticmethod
-    def _logger_info(sensor, data):
+    def _logger_info(data):
         _LOGGER.debug("Converting Logger info: '{}'".format(data))
+        sensor = {}
 
         if "CO2Factor" in data and "CO2Unit" in data:
             sensor["co2_factor"] = {
